<?php
/**
 * Widget API: Display Podcast from feed url class
 *
 * @package podcast-player
 * @since 1.0.0
 */

namespace Podcast_Player;

/**
 * Class used to display podcast episodes from a feed url.
 *
 * @since 1.0.0
 *
 * @see WP_Widget
 */
class Widget extends \WP_Widget {

	/**
	 * Holds all display styles.
	 *
	 * @since  1.0.0
	 * @access protected
	 * @var array
	 */
	protected $styles = [];

	/**
	 * Holds all display styles supported items.
	 *
	 * @since  1.0.0
	 * @access protected
	 * @var array
	 */
	protected $style_supported = [];

	/**
	 * Holds widget settings defaults, populated in constructor.
	 *
	 * @since  1.0.0
	 * @access protected
	 * @var array
	 */
	protected $defaults = [];

	/**
	 * Is this the premium version.
	 *
	 * @since  1.2.0
	 * @access protected
	 * @var    bool
	 */
	protected $is_premium = true;

	/**
	 * Sets up a new Blank widget instance.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		// Set widget instance settings default values.
		$this->defaults = [
			'title'               => '',
			'sortby'              => 'sort_date_desc',
			'filterby'            => '',
			'feed_url'            => '',
			'number'              => 10,
			'podcast_menu'        => '',
			'cover_image'         => '',
			'desc'                => '',
			'error'               => '',
			'pp_excerpt_length'   => 25,
			'pp_grid_columns'     => 3,
			'pp_header_default'   => '',
			'pp_list_default'     => '',
			'pp_hide_header'      => '',
			'pp_hide_cover'       => '',
			'pp_hide_title'       => '',
			'pp_hide_description' => '',
			'pp_hide_subscribe'   => '',
			'pp_hide_search'      => '',
			'pp_hide_author'      => '',
			'pp_hide_content'     => '',
			'pp_hide_loadmore'    => '',
			'pp_hide_download'    => '',
			'pp_hide_social'      => '',
			'pp_hide_featured'    => '',
			'pp_accent_color'     => '',
			'pp_display_style'    => '',
			'pp_aspect_ratio'     => 'squr',
			'pp_crop_method'      => 'centercrop',
			'pp_fetch_method'     => 'feed',
			'pp_post_type'        => 'post',
			'pp_taxonomy'         => '',
			'pp_terms'            => [],
			'pp_podtitle'         => '',
			'pp_audiosrc'         => '',
			'pp_audiotitle'       => '',
			'pp_audiolink'        => '',
			'pp_ahide_download'   => '',
			'pp_ahide_social'     => '',
			'pp_audio_message'    => '',
			'pp_play_frequency'   => 0,
			'pp_start_time'       => [ 0, 0, 0 ],
			'pp_start_when'       => 'start',
			'pp_msg_text'         => esc_html__( 'Episode will play after this message.', 'podcast-player' ),
			'pp_fonts'            => '',
			'pp_bgcolor'          => '',
			'pp_txtcolor'         => '',
			'pp_apple_sub'        => '',
			'pp_google_sub'       => '',
			'pp_elist'            => [ '' ],
			'pp_slist'            => [ '' ],
			'pp_seasons'          => '',
			'pp_episodes'         => '',
		];

		$this->is_premium = apply_filters( 'podcast_player_is_premium', false );

		// Set the widget options.
		$widget_ops = [
			'classname'                   => 'podcast_player',
			'description'                 => esc_html__( 'Create a podcast player widget.', 'podcast-player' ),
			'customize_selective_refresh' => true,
		];
		parent::__construct( 'podcast_player_widget', esc_html__( 'Podcast Player', 'podcast-player' ), $widget_ops );
	}

	/**
	 * Outputs the content for the current widget instance.
	 *
	 * @since 2.8.0
	 *
	 * @param array $args     Display arguments including 'before_title', 'after_title',
	 *                        'before_widget', and 'after_widget'.
	 * @param array $instance Settings for the current widget instance.
	 */
	public function widget( $args, $instance ) {

		$args['widget_id'] = isset( $args['widget_id'] ) ? $args['widget_id'] : $this->id;

		// Merge with defaults.
		$instance = wp_parse_args( (array) $instance, $this->defaults );

		$title = ! empty( $instance['title'] ) ? $instance['title'] : '';

		/** This filter is documented in wp-includes/widgets/class-wp-widget-pages.php */
		$title = apply_filters( 'widget_title', $title, $instance, $this->id_base );

		echo $args['before_widget']; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped

		if ( $title ) {
			echo $args['before_title'] . $title . $args['after_title']; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		}

		$display_args = apply_filters(
			'podcast_player_widget_display',
			[
				'url'              => $instance['feed_url'],
				'sortby'           => $instance['sortby'],
				'filterby'         => $instance['filterby'],
				'number'           => $instance['number'],
				'menu'             => $instance['podcast_menu'],
				'image'            => $instance['cover_image'],
				'description'      => $instance['desc'],
				'header-default'   => $instance['pp_header_default'],
				'list-default'     => $instance['pp_list_default'],
				'hide-header'      => $instance['pp_hide_header'],
				'hide-title'       => $instance['pp_hide_title'],
				'hide-cover-img'   => $instance['pp_hide_cover'],
				'hide-description' => $instance['pp_hide_description'],
				'hide-subscribe'   => $instance['pp_hide_subscribe'],
				'hide-search'      => $instance['pp_hide_search'],
				'hide-author'      => $instance['pp_hide_author'],
				'hide-content'     => $instance['pp_hide_content'],
				'hide-loadmore'    => $instance['pp_hide_loadmore'],
				'hide-download'    => $instance['pp_hide_download'],
				'hide-social'      => $instance['pp_hide_social'],
				'hide-featured'    => $instance['pp_hide_featured'],
				'accent-color'     => $instance['pp_accent_color'],
				'display-style'    => $instance['pp_display_style'],
				'apple-sub'        => $instance['pp_apple_sub'],
				'google-sub'       => $instance['pp_google_sub'],
				'from'             => 'widget',
			],
			$instance
		);

		podcast_player_display( $display_args, false );
		echo $args['after_widget']; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}

	/**
	 * Get array of all widget options.
	 *
	 * @param array $settings Array of settings for current widget instance.
	 *
	 * @since 1.0.0
	 */
	public function get_widget_options( $settings ) {
		$widget        = $this;
		$menus         = wp_get_nav_menus();
		$menu_arr      = wp_list_pluck( $menus, 'name', 'term_id' );
		$menu_arr      = [ '' => esc_html__( 'None', 'podcast-player' ) ] + $menu_arr;
		$display_style = $this->get_display_styles();

		return apply_filters(
			'podcast_player_widget_options',
			[
				'default' => [
					'title' => esc_html__( 'General Options', 'podcast-player' ),
					'items' => [
						'title'    => [
							'setting' => 'title',
							'label'   => esc_html__( 'Title', 'podcast-player' ),
							'type'    => 'text',
						],
						'feed_url' => [
							'setting'       => 'feed_url',
							'label'         => esc_html__( 'Podcast Feed URL', 'podcast-player' ),
							'type'          => 'text',
							'hide_callback' => function() use ( $widget, $settings ) {
								return $widget->is_option_not_equal( 'pp_fetch_method', 'feed', $settings );
							},
						],
					],
				],
				'info'    => [
					'title'       => esc_html__( 'Customize Podcast Content', 'podcast-player' ),
					'op_callback' => function() use ( $widget, $settings ) {
						return $widget->is_option_not_equal( 'pp_fetch_method', 'link', $settings );
					},
					'items'       => [
						'cover_image'   => [
							'setting' => 'cover_image',
							'label'   => esc_html__( 'Podcast Cover Image', 'podcast-player' ),
							'type'    => 'image_upload',
						],
						'desc'          => [
							'setting'     => 'desc',
							'label'       => esc_html__( 'Brief Description', 'podcast-player' ),
							'type'        => 'textarea',
							'input_attrs' => [
								'col' => 50,
								'row' => 3,
							],
						],
						'pp_apple_sub'  => [
							'setting' => 'pp_apple_sub',
							'label'   => esc_html__( 'Apple Podcast Subscription link.', 'podcast-player' ),
							'type'    => 'url',
						],
						'pp_google_sub' => [
							'setting' => 'pp_google_sub',
							'label'   => esc_html__( 'Google Podcast Subscription link.', 'podcast-player' ),
							'type'    => 'url',
						],
						'podcast_menu'  => [
							'setting' => 'podcast_menu',
							'label'   => esc_html__( 'Custom menu for more subscription links', 'podcast-player' ),
							'type'    => 'select',
							'choices' => $menu_arr,
						],
						'number'        => [
							'setting'     => 'number',
							'label'       => esc_html__( 'Number of episodes to show at a time.', 'podcast-player' ),
							'type'        => 'number',
							'input_attrs' => [
								'step' => 1,
								'min'  => 1,
								'size' => 3,
							],
						],
					],
				],
				'show'    => [
					'title'       => esc_html__( 'Show/Hide Player Items', 'podcast-player' ),
					'op_callback' => function() use ( $widget, $settings ) {
						return $widget->is_option_not_equal( 'pp_fetch_method', 'link', $settings );
					},
					'items'       => [
						'pp_header_default'   => [
							'setting'       => 'pp_header_default',
							'label'         => esc_html__( 'Show Podcast Header by Default.', 'podcast-player' ),
							'type'          => 'checkbox',
							'hide_callback' => function() use ( $widget, $settings ) {
								return $widget->is_option_not_equal( 'pp_display_style', '', $settings ) && $widget->is_option_not_equal( 'pp_display_style', 'legacy', $settings );
							},
						],
						'pp_list_default'     => [
							'setting'       => 'pp_list_default',
							'label'         => esc_html__( 'Show episodes list by default on mini player.', 'podcast-player' ),
							'type'          => 'checkbox',
							'hide_callback' => function() use ( $widget, $settings ) {
								return $widget->is_option_not_equal( 'pp_display_style', '', $settings ) && $widget->is_option_not_equal( 'pp_display_style', 'legacy', $settings );
							},
						],
						'pp_hide_header'      => [
							'setting' => 'pp_hide_header',
							'label'   => esc_html__( 'Hide Podcast Header Information.', 'podcast-player' ),
							'type'    => 'checkbox',
						],
						'pp_hide_cover'       => [
							'setting'       => 'pp_hide_cover',
							'label'         => esc_html__( 'Hide cover image.', 'podcast-player' ),
							'type'          => 'checkbox',
							'hide_callback' => function() use ( $widget, $settings ) {
								return $widget->is_option_equal( 'pp_hide_header', 'yes', $settings );
							},
						],
						'pp_hide_title'       => [
							'setting'       => 'pp_hide_title',
							'label'         => esc_html__( 'Hide Podcast Title.', 'podcast-player' ),
							'type'          => 'checkbox',
							'hide_callback' => function() use ( $widget, $settings ) {
								return $widget->is_option_equal( 'pp_hide_header', 'yes', $settings );
							},
						],
						'pp_hide_description' => [
							'setting'       => 'pp_hide_description',
							'label'         => esc_html__( 'Hide Podcast Description.', 'podcast-player' ),
							'type'          => 'checkbox',
							'hide_callback' => function() use ( $widget, $settings ) {
								return $widget->is_option_equal( 'pp_hide_header', 'yes', $settings );
							},
						],
						'pp_hide_subscribe'   => [
							'setting'       => 'pp_hide_subscribe',
							'label'         => esc_html__( 'Hide Custom menu.', 'podcast-player' ),
							'type'          => 'checkbox',
							'hide_callback' => function() use ( $widget, $settings ) {
								return $widget->is_option_equal( 'pp_hide_header', 'yes', $settings );
							},
						],
						'pp_hide_search'      => [
							'setting' => 'pp_hide_search',
							'label'   => esc_html__( 'Hide Podcast Search.', 'podcast-player' ),
							'type'    => 'checkbox',
						],
						'pp_hide_author'      => [
							'setting' => 'pp_hide_author',
							'label'   => esc_html__( 'Hide Episode Author/Podcaster Name.', 'podcast-player' ),
							'type'    => 'checkbox',
						],
						'pp_hide_content'     => [
							'setting'       => 'pp_hide_content',
							'label'         => esc_html__( 'Hide Episode Text Content/Transcript.', 'podcast-player' ),
							'type'          => 'checkbox',
							'hide_callback' => function() use ( $widget, $settings ) {
								return $widget->is_premium && $widget->is_option_not_equal( 'pp_fetch_method', 'feed', $settings );
							},
						],
						'pp_hide_loadmore'    => [
							'setting' => 'pp_hide_loadmore',
							'label'   => esc_html__( 'Hide Load More Episodes Button.', 'podcast-player' ),
							'type'    => 'checkbox',
						],
						'pp_hide_download'    => [
							'setting' => 'pp_hide_download',
							'label'   => esc_html__( 'Hide Episode Download Link.', 'podcast-player' ),
							'type'    => 'checkbox',
						],
						'pp_hide_social'      => [
							'setting' => 'pp_hide_social',
							'label'   => esc_html__( 'Hide Social Share Links.', 'podcast-player' ),
							'type'    => 'checkbox',
						],
						'pp_hide_featured'    => [
							'setting' => 'pp_hide_featured',
							'label'   => esc_html__( 'Hide Episodes Featured Image.', 'podcast-player' ),
							'type'    => 'checkbox',
						],
					],
				],
				'style'   => [
					'title' => esc_html__( 'Podcast Player Styling', 'podcast-player' ),
					'items' => [
						'pp_accent_color'  => [
							'setting' => 'pp_accent_color',
							'label'   => esc_html__( 'Accent Color', 'podcast-player' ),
							'type'    => 'color',
						],
						'pp_display_style' => [
							'setting'       => 'pp_display_style',
							'label'         => esc_html__( 'Podcast Player Display Style', 'podcast-player' ),
							'type'          => 'select',
							'choices'       => $display_style,
							'hide_callback' => function() use ( $widget, $settings ) {
								return $widget->is_option_equal( 'pp_fetch_method', 'link', $settings );
							},
						],
					],
				],
				'filter'  => [
					'title'       => esc_html__( 'Sort & Filter Options', 'podcast-player' ),
					'op_callback' => function() use ( $widget, $settings ) {
						return $widget->is_option_not_equal( 'pp_fetch_method', 'link', $settings );
					},
					'items'       => [
						'sortby'   => [
							'setting' => 'sortby',
							'label'   => esc_html__( 'Sort Podcast Episodes By', 'podcast-player' ),
							'type'    => 'select',
							'choices' => [
								'sort_title_desc' => esc_html__( 'Title Descending', 'podcast-player' ),
								'sort_title_asc'  => esc_html__( 'Title Ascending', 'podcast-player' ),
								'sort_date_desc'  => esc_html__( 'Date Descending', 'podcast-player' ),
								'sort_date_asc'   => esc_html__( 'Date Ascending', 'podcast-player' ),
							],
						],
						'filterby' => [
							'setting' => 'filterby',
							'label'   => esc_html__( 'Show episodes only if title contains following', 'podcast-player' ),
							'type'    => 'text',
						],
					],
				],
			],
			$widget,
			$settings
		);
	}

	/**
	 * Outputs the settings form for the widget.
	 *
	 * @since 1.0.0
	 *
	 * @param array $instance Current settings.
	 */
	public function form( $instance ) {
		// Merge with defaults.
		$instance = wp_parse_args( (array) $instance, $this->defaults );
		$options  = $this->get_widget_options( $instance );

		$default_markup = '';
		$options_markup = '';
		foreach ( $options as $option => $args ) {
			$items  = $args['items'];
			$showop = isset( $args['op_callback'] ) && is_callable( $args['op_callback'] ) ? call_user_func( $args['op_callback'] ) : true;
			$markup = '';
			foreach ( $items as $item => $attr ) {
				$dcall = isset( $attr['display_callback'] ) && is_callable( $attr['display_callback'] ) ? call_user_func( $attr['display_callback'] ) : true;
				if ( ! $dcall ) {
					continue;
				}

				$set   = $attr['setting'];
				$id    = esc_attr( $this->get_field_id( $set ) );
				$name  = esc_attr( $this->get_field_name( $set ) );
				$type  = $attr['type'];
				$label = isset( $attr['label'] ) ? $attr['label'] : '';
				$desc  = isset( $attr['desc'] ) ? $attr['desc'] : '';
				$iatt  = isset( $attr['input_attrs'] ) ? $attr['input_attrs'] : [];
				$hcal  = isset( $attr['hide_callback'] ) && is_callable( $attr['hide_callback'] ) ? call_user_func( $attr['hide_callback'] ) : false;

				$inputattr = '';
				foreach ( $iatt as $att => $val ) {
					$inputattr .= esc_html( $att ) . '="' . esc_attr( $val ) . '" ';
				}

				switch ( $type ) {
					case 'select':
						$optmar  = $this->label( $set, $label, false );
						$optmar .= $this->select( $set, $attr['choices'], $instance[ $set ], [], false );
						break;
					case 'checkbox':
						$optmar  = sprintf( '<input name="%s" id="%s" type="checkbox" value="yes" %s />', $name, $id, checked( $instance[ $set ], 'yes', false ) );
						$optmar .= $this->label( $set, $label, false );
						break;
					case 'text':
						$optmar  = $this->label( $set, $label, false );
						$optmar .= sprintf( '<input class="widefat" name="%1$s" id="%2$s" type="text" value="%3$s" />', $name, $id, esc_attr( $instance[ $set ] ) );
						$optmar .= sprintf( '<div class="pp-desc">%s</div>', $desc );
						break;
					case 'url':
						$optmar  = $this->label( $set, $label, false );
						$optmar .= sprintf( '<input class="widefat" name="%1$s" id="%2$s" type="url" value="%3$s" />', $name, $id, esc_attr( $instance[ $set ] ) );
						$optmar .= sprintf( '<div class="pp-desc">%s</div>', $desc );
						break;
					case 'number':
						$optmar  = $this->label( $set, $label, false );
						$optmar .= sprintf( '<input class="widefat" name="%1$s" id="%2$s" type="number" value="%3$s" %4$s />', $name, $id, absint( $instance[ $set ] ), $inputattr );
						$optmar .= sprintf( '<div class="pp-desc">%s</div>', $desc );
						break;
					case 'mmss':
						$optmar  = $this->label( $set, $label, false );
						$optmar .= sprintf( '<div class="mmss-time" id="%s">', $id );
						$optmar .= sprintf( '<input class="tiny-text" name="%1$s[]" type="number" value="%2$s" min="0" max="10" size="2"/> : ', $name, absint( $instance[ $set ][0] ) );
						$optmar .= sprintf( '<input class="tiny-text" name="%1$s[]" type="number" value="%2$s"  min="0" max="59" size="2" /> : ', $name, absint( $instance[ $set ][1] ) );
						$optmar .= sprintf( '<input class="tiny-text" name="%1$s[]" type="number" value="%2$s"  min="0" max="59" size="2" />', $name, absint( $instance[ $set ][2] ) );
						$optmar .= '</div>';
						$optmar .= sprintf( '<div class="pp-desc">%s</div>', $desc );
						break;
					case 'textarea':
						$optmar  = $this->label( $set, $label, false );
						$optmar .= sprintf( '<textarea class="widefat" name="%1$s" id="%2$s" %3$s >%4$s</textarea>', $name, $id, $inputattr, esc_attr( $instance[ $set ] ) );
						break;
					case 'image_upload':
						$optmar  = $this->label( $set, $label, false );
						$optmar .= $this->image_upload( $id, $name, $instance[ $set ] );
						break;
					case 'color':
						$optmar  = $this->label( $set, $label, false );
						$optmar .= sprintf( '<input class="pp-color-picker" name="%1$s" id="%2$s" type="text" value="%3$s" />', $name, $id, esc_attr( sanitize_hex_color( $instance[ $set ] ) ) );
						break;
					case 'taxonomy':
						$optmar = $this->taxonomies_select( $instance['pp_post_type'], $instance['pp_taxonomy'] );
						break;
					case 'terms':
						$optmar = $this->terms_checklist( $instance['pp_taxonomy'], $instance['pp_terms'] );
						break;
					case 'elist':
						$optmar = $this->episodes_checklist( $instance );
						break;
					case 'slist':
						$optmar = $this->seasons_checklist( $instance );
						break;
					default:
						$optmar = apply_filters( 'podcast_player_custom_option_field', false, $item, $attr, $this, $instance );
						break;
				}
				$style   = $hcal ? 'style="display: none;"' : '';
				$markup .= $optmar ? sprintf( '<div class="%1$s pp-widget-option" %2$s>%3$s</div>', $set, $style, $optmar ) : '';
			}
			if ( 'default' === $option ) {
				$default_markup = $markup;
			} else {
				$opstyle         = $showop ? '' : 'style="display: none;"';
				$section         = sprintf( '<a class="pp-settings-toggle pp-%1$s-toggle" %2$s>%3$s</a>', $option, $opstyle, $args['title'] );
				$section        .= sprintf( '<div class="pp-settings-content pp-%1$s-content">%2$s</div>', $option, $markup );
				$options_markup .= $section;
			}
		}
		printf( '%1$s<div class="pp-options-wrapper">%2$s</div>', $default_markup, $options_markup ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		if ( 0 < strlen( $instance['error'] ) ) :
			?>
			<div style="color: red; font-weight: bold;"><?php echo esc_html( $instance['error'] ); ?></div>
			<?php
		endif;
	}

	/**
	 * Handles updating the settings for the current widget instance.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $new_instance New settings for this instance as input by the user via
	 *                            WP_Widget::form().
	 * @param array $old_instance Old settings for this instance.
	 * @return array Updated settings to save.
	 */
	public function update( $new_instance, $old_instance ) {

		// Merge with defaults.
		$new_instance = wp_parse_args( (array) $new_instance, $this->defaults );

		$instance                = $old_instance;
		$img_id                  = absint( $new_instance['cover_image'] );
		$img_url                 = $img_id ? wp_get_attachment_image_src( $img_id ) : false;
		$instance['cover_image'] = $img_url ? $img_id : '';

		$sanitize_int = [
			'number',
			'podcast_menu',
		];
		foreach ( $sanitize_int as $setting ) {
			$instance[ $setting ] = absint( $new_instance[ $setting ] );
		}

		$sanitize_text = [
			'title',
			'desc',
			'sortby',
			'filterby',
			'pp_display_style',
		];
		foreach ( $sanitize_text as $setting ) {
			$instance[ $setting ] = sanitize_text_field( $new_instance[ $setting ] );
		}

		$sanitize_url = [
			'pp_apple_sub',
			'pp_google_sub',
		];
		foreach ( $sanitize_url as $setting ) {
			$instance[ $setting ] = esc_url_raw( $new_instance[ $setting ] );
		}

		$sanitize_bool = [
			'pp_list_default',
			'pp_hide_title',
			'pp_hide_cover',
			'pp_hide_description',
			'pp_header_default',
			'pp_hide_header',
			'pp_hide_subscribe',
			'pp_hide_search',
			'pp_hide_author',
			'pp_hide_content',
			'pp_hide_loadmore',
			'pp_hide_download',
			'pp_hide_social',
			'pp_hide_featured',
		];
		foreach ( $sanitize_bool as $setting ) {
			$instance[ $setting ] = ( 'yes' === $new_instance[ $setting ] ) ? 'yes' : '';
		}

		$instance['pp_accent_color'] = sanitize_hex_color( $new_instance['pp_accent_color'] );

		if ( $this->is_premium && isset( $instance['pp_fetch_method'] ) && 'feed' !== $instance['pp_fetch_method'] ) {
			$instance['feed_url'] = '';
			$instance['error']    = '';
		} else {
			$error   = '';
			$feedurl = '';
			if ( isset( $old_instance['feed_url'] ) && ( $new_instance['feed_url'] === $old_instance['feed_url'] ) ) {
				$feedurl = $old_instance['feed_url'];
			} elseif ( $new_instance['feed_url'] ) {
				$validurl = podcast_player_get_valid_url( $new_instance['feed_url'] );
				if ( is_int( $validurl ) ) {
					$error = podcast_player_error_msg( $validurl );
				}

				$feedurl = $new_instance['feed_url'];
				// Check if a valid url has been provided.
				if ( false !== strpos( $feedurl, 'http://' ) || false !== strpos( $feedurl, 'https://' ) ) {
					$feedurl = esc_url_raw( wp_strip_all_tags( $feedurl ) );
				} else {
					if ( is_int( $validurl ) && ! in_array( $validurl, [ 1, 2 ], true ) ) {
						$feedurl = sanitize_text_field( $feedurl );
					}
				}
			}

			$instance['feed_url'] = $feedurl;
			$instance['error']    = sanitize_text_field( $error );
		}

		return apply_filters( 'podcast_player_widget_update', $instance, $new_instance, $this );
	}

	/**
	 * Check if widget setting contains a particular value.
	 *
	 * @param str   $setting Setting to be checked.
	 * @param str   $val Setting value to be matched.
	 * @param array $settings Array of settings for current widget instance.
	 * @return bool
	 */
	public function is_option_equal( $setting, $val, $settings ) {
		return isset( $settings[ $setting ] ) && $val === $settings[ $setting ];
	}

	/**
	 * Check if widget setting doen not contains a particular value.
	 *
	 * @param str   $setting Setting to be checked.
	 * @param str   $val Setting value to be matched.
	 * @param array $settings Array of settings for current widget instance.
	 * @return bool
	 */
	public function is_option_not_equal( $setting, $val, $settings ) {
		return ! isset( $settings[ $setting ] ) || $val !== $settings[ $setting ];
	}

	/**
	 * Markup for 'label' for widget input options.
	 *
	 * @param str  $for  Label for which ID.
	 * @param str  $text Label text.
	 * @param bool $echo Display or Return.
	 * @return void|string
	 */
	public function label( $for, $text, $echo = true ) {
		$label = sprintf( '<label for="%s">%s</label>', esc_attr( $this->get_field_id( $for ) ), esc_html( $text ) );
		if ( $echo ) {
			echo $label; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		} else {
			return $label;
		}
	}

	/**
	 * Markup for Select dropdown lists for widget options.
	 *
	 * @param str   $for      Select for which ID.
	 * @param array $options  Select options as 'value => label' pair.
	 * @param str   $selected selected option.
	 * @param array $classes  Options HTML classes.
	 * @param bool  $echo     Display or return.
	 * @return void|string
	 */
	public function select( $for, $options, $selected, $classes = [], $echo = true ) {
		$select      = '';
		$final_class = '';
		foreach ( $options as $value => $label ) {
			if ( isset( $classes[ $value ] ) ) {
				$option_classes = (array) $classes[ $value ];
				$option_classes = array_map( 'esc_attr', $option_classes );
				$final_class    = 'class="' . join( ' ', $option_classes ) . '"';
			}
			$select .= sprintf( '<option value="%1$s" %2$s %3$s>%4$s</option>', esc_attr( $value ), $final_class, selected( $value, $selected, false ), esc_html( $label ) );
		}

		$select = sprintf(
			'<select id="%1$s" name="%2$s" class="podcast-player-%3$s widefat">%4$s</select>',
			esc_attr( $this->get_field_id( $for ) ),
			esc_attr( $this->get_field_name( $for ) ),
			esc_attr( str_replace( '_', '-', $for ) ),
			$select
		);

		if ( $echo ) {
			echo $select; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		} else {
			return $select;
		}
	}

	/**
	 * Image upload option markup.
	 *
	 * @since 1.0.0
	 *
	 * @param str $id      Field ID.
	 * @param str $name    Field Name.
	 * @param int $value   Uploaded image id.
	 * @return str Widget form image upload markup.
	 */
	public function image_upload( $id, $name, $value ) {

		$value          = absint( $value );
		$uploader_class = '';
		$class          = 'podcast-player-hidden';

		if ( $value ) {
			$image_src = wp_get_attachment_image_src( $value, 'large' );
			if ( $image_src ) {
				$featured_markup = sprintf( '<img class="custom-widget-thumbnail" src="%s">', esc_url( $image_src[0] ) );
				$class           = '';
				$uploader_class  = 'has-image';
			} else {
				$featured_markup = esc_html__( 'Podcast Cover Image', 'podcast-player' );
			}
		} else {
			$featured_markup = esc_html__( 'Podcast Cover Image', 'podcast-player' );
		}

		$markup  = sprintf( '<a class="podcast-player-widget-img-uploader %s">%s</a>', $uploader_class, $featured_markup );
		$markup .= sprintf( '<span class="podcast-player-widget-img-instruct %s">%s</span>', $class, esc_html__( 'Click the image to edit/update', 'podcast-player' ) );
		$markup .= sprintf( '<a class="podcast-player-widget-img-remover %s">%s</a>', $class, esc_html__( 'Remove Featured Image', 'podcast-player' ) );
		$markup .= sprintf( '<input class="podcast-player-widget-img-id" name="%s" id="%s" value="%s" type="hidden" />', $name, $id, $value );
		return $markup;
	}

	/**
	 * Prints select list of all taxonomies for a post type.
	 *
	 * @param str   $post_type Selected post type.
	 * @param array $selected  Selected taxonomy in widget form.
	 */
	public function taxonomies_select( $post_type, $selected = [] ) {

		$options = pp_pro_get_taxonomies();

		// Get HTML classes for select options.
		$taxonomies = get_taxonomies( [], 'objects' );
		$classes    = wp_list_pluck( $taxonomies, 'object_type', 'name' );
		if ( $post_type && 'page' !== $post_type ) {
			foreach ( $classes as $name => $type ) {
				$type = (array) $type;
				if ( ! in_array( $post_type, $type, true ) ) {
					$type[]           = 'podcast-player-hidden';
					$classes[ $name ] = $type;
				}
			}
		}
		$classes[''] = 'always-visible';

		$markup = '';
		// Taxonomy Select markup.
		$markup .= $this->label( 'pp_taxonomy', esc_html__( 'Get Episodes by Taxonomy', 'podcast-player' ), false );
		$markup .= $this->select( 'pp_taxonomy', $options, $selected, $classes, false );
		return $markup;
	}

	/**
	 * Prints a checkbox list of all terms for a taxonomy.
	 *
	 * @param str   $taxonomy       Selected Taxonomy.
	 * @param array $selected_terms Selected Terms.
	 */
	public function terms_checklist( $taxonomy, $selected_terms = [] ) {

		$tax = pp_pro_get_taxonomies();
		$tax = array_filter( array_keys( $tax ) );

		// Get list of all registered terms.
		$terms = get_terms( [ 'taxonomy' => $tax ] );

		// Get 'checkbox' options as value => label.
		$options = wp_list_pluck( $terms, 'name', 'slug' );

		// Get HTML classes for checkbox options.
		$classes = wp_list_pluck( $terms, 'taxonomy', 'slug' );
		if ( $taxonomy ) {
			foreach ( $classes as $slug => $taxon ) {
				if ( $taxonomy !== $taxon ) {
					$classes[ $slug ] .= ' podcast-player-hidden';
				}
			}
		}

		$markup = '';
		// Terms Checkbox markup.
		$markup .= $this->label( 'pp_terms', esc_html__( 'Select Terms', 'podcast-player' ), false );
		$markup .= $this->mu_checkbox( 'pp_terms', $options, $selected_terms, $classes, false );
		return $markup;
	}

	/**
	 * Prints a checkbox list of all episodes of current podcast feed.
	 *
	 * @param arr $args Settings for current podcast player instance.
	 */
	public function episodes_checklist( $args ) {

		$method   = isset( $args['pp_fetch_method'] ) ? $args['pp_fetch_method'] : 'feed';
		$options  = [];
		$classes  = [];
		$selected = isset( $args['pp_elist'] ) ? $args['pp_elist'] : [ '' ];

		$options[''] = esc_html__( 'Show All Episodes', 'podcast-player' );
		$is_hidden   = true;
		if ( 'feed' === $method ) {
			$url       = isset( $args['feed_url'] ) ? $args['feed_url'] : '';
			$ops       = apply_filters( 'podcast_player_feed_elist', $url, [] );
			$ops       = isset( $ops['episodes'] ) ? $ops['episodes'] : [];
			$is_hidden = empty( $ops ) ? true : false;
			$options   = array_merge( $options, $ops );
		} elseif ( 'post' === $method ) {
			$sets = [
				'post-type' => isset( $args['pp_post_type'] ) ? $args['pp_post_type'] : 'post',
				'taxonomy'  => isset( $args['pp_taxonomy'] ) ? $args['pp_taxonomy'] : '',
				'terms'     => isset( $args['pp_terms'] ) ? $args['pp_terms'] : [],
				'sortby'    => isset( $args['sortby'] ) ? $args['sortby'] : 'sort_date_desc',
				'filterby'  => isset( $args['filterby'] ) ? $args['filterby'] : '',
				'number'    => -1,
			];

			$ops       = apply_filters( 'podcast_player_posts_elist', $sets );
			$ops       = isset( $ops['episodes'] ) ? $ops['episodes'] : [];
			$is_hidden = empty( $ops ) ? true : false;
			$options   = array_merge( $options, $ops );
		}

		foreach ( $options as $id => $label ) {
			if ( '' !== $id ) {
				$classes[ $id ] = 'pp-episodes';
			} else {
				$classes[ $id ] = 'd-episode';
			}
		}

		$markup = '';
		$style  = $is_hidden ? 'style="display: none;"' : '';
		// Episodes list Checkbox markup.
		$markup .= $this->label( 'pp_elist', esc_html__( 'Select Episodes to be displayed', 'podcast-player' ), false );
		$markup .= $this->mu_checkbox( 'pp_elist', $options, $selected, $classes, false );
		return sprintf( '<div class="pp-episodes-list" %1$s>%2$s</div>', $style, $markup );
	}

	/**
	 * Prints a checkbox list of all seasons of current podcast feed.
	 *
	 * @param arr $args Settings for current podcast player instance.
	 */
	public function seasons_checklist( $args ) {

		$method    = isset( $args['pp_fetch_method'] ) ? $args['pp_fetch_method'] : 'feed';
		$options   = [];
		$classes   = [];
		$selected  = isset( $args['pp_slist'] ) ? $args['pp_slist'] : [ '' ];
		$is_hidden = true;

		$options[''] = esc_html__( 'Show All Seasons', 'podcast-player' );

		if ( 'feed' === $method ) {
			$url       = isset( $args['feed_url'] ) ? $args['feed_url'] : '';
			$ops       = apply_filters( 'podcast_player_feed_slist', $url, [] );
			$ops       = isset( $ops['seasons'] ) ? $ops['seasons'] : [];
			$is_hidden = empty( $ops ) ? true : false;
			$options   = array_merge( $options, $ops );
		}

		foreach ( $options as $id => $label ) {
			if ( '' !== $id ) {
				$classes[ $id ] = 'pp-seasons';
			} else {
				$classes[ $id ] = 'd-season';
			}
		}

		$markup = '';
		$style  = $is_hidden ? 'style="display: none;"' : '';
		// Episodes list Checkbox markup.
		$markup .= $this->label( 'pp_slist', esc_html__( 'Select seasons to be displayed', 'podcast-player' ), false );
		$markup .= $this->mu_checkbox( 'pp_slist', $options, $selected, $classes, false );
		return sprintf( '<div class="pp-seasons-list" %1$s>%2$s</div>', $style, $markup );
	}

	/**
	 * Markup for multiple checkbox for widget options.
	 *
	 * @param str   $for      Select for which ID.
	 * @param array $options  Select options as 'value => label' pair.
	 * @param str   $selected selected option.
	 * @param array $classes  Checkbox input HTML classes.
	 * @param bool  $echo     Display or return.
	 * @return void|string
	 */
	public function mu_checkbox( $for, $options, $selected = [], $classes = [], $echo = true ) {

		$final_class = '';

		$mu_checkbox = '<div class="' . esc_attr( $for ) . '-checklist"><ul id="' . esc_attr( $this->get_field_id( $for ) ) . '">';

		$selected    = array_map( 'strval', $selected );
		$rev_options = $options;

		// Moving selected items on top of the array.
		foreach ( $options as $id => $label ) {
			if ( in_array( strval( $id ), $selected, true ) ) {
				$rev_options = [ $id => $label ] + $rev_options;
			}
		}

		// Bring default option at top.
		if ( isset( $rev_options[''] ) ) {
			$rev_options = [ '' => $rev_options[''] ] + $rev_options;
		}

		foreach ( $rev_options as $id => $label ) {
			if ( isset( $classes[ $id ] ) ) {
				$final_class = ' class="' . esc_attr( $classes[ $id ] ) . '"';
			}
			$mu_checkbox .= "\n<li$final_class>" . '<label class="selectit"><input value="' . esc_attr( $id ) . '" type="checkbox" name="' . esc_attr( $this->get_field_name( $for ) ) . '[]"' . checked( in_array( strval( $id ), $selected, true ), true, false ) . ' /><span class="cblabel">' . esc_html( $label ) . "</span></label></li>\n";
		}
		$mu_checkbox .= "</ul></div>\n";

		if ( $echo ) {
			echo $mu_checkbox; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		} else {
			return $mu_checkbox;
		}
	}

	/**
	 * Get display styles.
	 *
	 * @return array
	 */
	public function get_display_styles() {
		if ( ! empty( $this->styles ) ) {
			return $this->styles;
		}

		$styles = podcast_player_default_styles();
		foreach ( $styles as $style => $args ) {
			$this->styles[ $style ]          = $args['label'];
			$this->style_supported[ $style ] = $args['support'];
		}

		return $this->styles;
	}

	/**
	 * Check if item is supported by the style.
	 *
	 * @param string $style Current display style.
	 * @param string $item  item to be checked for support.
	 * @return bool
	 */
	public function is_style_support( $style, $item ) {
		if ( ! $style && '' !== $style ) {
			return false;
		}

		$sup_arr = $this->style_supported[ $style ];
		return in_array( $item, $sup_arr, true );
	}
}
