<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.vedathemes.com
 * @since      1.0.0
 *
 * @package    Podcast_Player
 * @subpackage Podcast_Player/Frontend
 */

namespace Podcast_Player\Frontend;

use Podcast_Player\Frontend\Inc\Loader;
use Podcast_Player\Frontend\Inc\Feed;

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Podcast_Player
 * @subpackage Podcast_Player/Frontend
 * @author     vedathemes <contact@vedathemes.com>
 */
class Register {

	/**
	 * Holds the instance of this class.
	 *
	 * @since  1.0.0
	 * @access protected
	 * @var    object
	 */
	protected static $instance = null;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {}

	/**
	 * Register hooked functions.
	 *
	 * @since 1.0.0
	 */
	public static function init() {

		// Instantiate front-end loader class.
		$loader = Loader::get_instance();

		// Load front-end scripts, styles and icons.
		self::load_resources( $loader );

		// Add media element player settings.
		self::register_mejs_settings( $loader );

		// Add Elementor preview screen support.
		self::elementor_support( $loader );

		// Instantiate front-end feed class to register JS hooks.
		$feed = Feed::get_instance();

		// Make pp data available to front-end scripts.
		self::register_script_data( $feed );

		// Support Ajax loading functionality.
		self::support_ajax_functionality( $feed );
	}

	/**
	 * Load front-end scripts and styles.
	 *
	 * @since 3.3.0
	 *
	 * @param object $instance PP front loader instance.
	 */
	public static function load_resources( $instance ) {

		// The script must be loaded before mediaelement-migrate script.
		add_action( 'wp_enqueue_scripts', array( $instance, 'mm_error_fix' ), 9999 );

		// Load other scripts and styles for podcast player.
		add_action( 'wp_footer', array( $instance, 'enqueue_resources' ) );

		// Load svg icons to be used in podcast player.
		add_action( 'wp_footer', array( $instance, 'svg_icons' ), 9999 );
	}

	/**
	 * Register Media Element player settings and data..
	 *
	 * @since 3.3.0
	 *
	 * @param object $instance PP front loader instance.
	 */
	public static function register_mejs_settings( $instance ) {
		add_filter( 'podcast_player_mediaelement_settings', array( $instance, 'mejs_settings' ) );
	}

	/**
	 * Add Elementor preview screen support.
	 *
	 * @since 3.3.0
	 *
	 * @param object $instance PP front loader instance.
	 */
	public static function elementor_support( $instance ) {
		if (
			in_array(
				'elementor/elementor.php',
				apply_filters( 'active_plugins', get_option( 'active_plugins' ) ),
				true
			)
		) {
			add_action(
				'elementor/preview/enqueue_scripts',
				array( $instance, 'enqueue_elementor_resources' )
			);
		}
	}

	/**
	 * Make pp data available to front-end scripts.
	 *
	 * @since 3.3.0
	 *
	 * @param object $instance PP feed instance.
	 */
	public static function register_script_data( $instance ) {
		add_filter( 'podcast_player_script_data', array( $instance, 'scripts_data' ) );
	}

	/**
	 * Support Front-end ajax loading functionality.
	 *
	 * @since 3.3.0
	 *
	 * @param object $instance PP feed instance.
	 */
	public static function support_ajax_functionality( $instance ) {

		// Handle Ajax request made to fetch next set of episodes.
		add_action( 'wp_ajax_pp_fetch_episodes', array( $instance, 'fetch_episodes' ) );
		add_action( 'wp_ajax_nopriv_pp_fetch_episodes', array( $instance, 'fetch_episodes' ) );

		// Handle Ajax request to search specific episodes.
		add_action( 'wp_ajax_pp_search_episodes', array( $instance, 'search_episodes' ) );
		add_action( 'wp_ajax_nopriv_pp_search_episodes', array( $instance, 'search_episodes' ) );
	}
}
